"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const path = require("path");
const uuid = require("uuid/v5");
const DialogflowDefault_1 = require("./DialogflowDefault");
const Schema_1 = require("./Schema");
const NAMESPACE = "dialogflow";
const AVAILABLE_LOCALES = [
    "en",
    "de",
    "fr",
    "ja",
    "ko",
    "es",
    "pt",
    "it",
    "ru",
    "hi",
    "th",
    "id",
    "da",
    "no",
    "nl",
    "sv",
    "en-US",
    "en-AU",
    "en-CA",
    "en-IN",
    "en-GB",
    "de-DE",
    "fr-FR",
    "fr-CA",
    "ja-JP",
    "ko-KR",
    "es-ES",
    "es-MX",
    "pt-BR",
    "it-IT",
    "ru-RU",
    "hi-IN",
    "th-TH",
    "id-ID",
    "da-DK",
    "no-NO",
    "nl-NL",
    "sv-SE"
];
class DialogflowSchema extends Schema_1.Schema {
    constructor(voxaSheets, interactionOptions) {
        super(NAMESPACE, AVAILABLE_LOCALES, voxaSheets, interactionOptions);
        this.environment = "staging";
        this.builtIntents = [];
    }
    validate() { }
    build(locale, environment) {
        this.buildIntent(locale, environment);
        this.buildUtterances(locale, environment);
        this.buildEntities(locale, environment);
        this.buildPackage(environment);
        this.buildAgent(locale, environment);
    }
    buildPackage(environment) {
        const file = {
            path: path.join(this.interactionOptions.rootPath, this.interactionOptions.speechPath, this.NAMESPACE, environment, "package.json"),
            content: {
                version: "1.0.0"
            }
        };
        this.fileContent.push(file);
    }
    getLocale(locale) {
        return locale.split("-")[0];
    }
    buildAgent(locale, environment) {
        const intents = this.builtIntents;
        const invocation = _.find(this.invocations, { locale, environment });
        const invocationName = _.get(invocation, "name", "Skill with no name");
        const intentsByPlatformAndEnvironments = this.intentsByPlatformAndEnvironments(locale, environment);
        const startIntents = _.chain(intentsByPlatformAndEnvironments)
            .filter({ startIntent: true })
            .map(intent => {
            const intentName = intent.name.replace("AMAZON.", "");
            const startIntent = _.chain(intents)
                .find(i => i.name === intent.name || i.name === intentName)
                .value();
            if (startIntent) {
                return {
                    intentId: _.get(startIntent, "id"),
                    signInRequired: !!_.get(startIntent, "signInRequired")
                };
            }
        })
            .compact()
            .value();
        const endIntentIds = _.chain(intentsByPlatformAndEnvironments)
            .filter({ endIntent: true })
            .map(intent => {
            const intentName = intent.name.replace("AMAZON.", "");
            const intentId = _.chain(intents)
                .find(i => i.name === intent.name || i.name === intentName)
                .get("id")
                .value();
            return intentId;
        })
            .compact()
            .value();
        const supportedLanguages = _(this.invocations)
            .filter({ environment })
            .map("locale")
            .uniq()
            .value();
        const language = this.getLocale(locale);
        const agent = _.merge(_.cloneDeep(DialogflowDefault_1.AGENT), _.cloneDeep(this.mergeManifest(environment)), _.cloneDeep({
            description: invocationName,
            language,
            supportedLanguages,
            googleAssistant: { project: _.kebabCase(invocationName), startIntents, endIntentIds }
        }));
        const file = {
            path: path.join(this.interactionOptions.rootPath, this.interactionOptions.speechPath, this.NAMESPACE, environment, "agent.json"),
            content: agent
        };
        this.fileContent.push(file);
    }
    buildUtterances(locale, environment) {
        const intentsByPlatformAndEnvironments = this.intentsByPlatformAndEnvironments(locale, environment);
        locale = this.getLocale(locale);
        const intents = intentsByPlatformAndEnvironments.map((rawIntent) => {
            let { name, samples, events } = rawIntent;
            const { slotsDefinition } = rawIntent;
            name = name.replace("AMAZON.", "");
            const builtInIntentSamples = _.get(DialogflowDefault_1.BUILT_IN_INTENTS, name, []);
            samples = _.chain(samples)
                .concat(builtInIntentSamples)
                .uniq()
                .value();
            events = name === "LaunchIntent" ? ["WELCOME", "GOOGLE_ASSISTANT_WELCOME"] : events;
            events = events.map((eventName) => ({ name: eventName }));
            const parameters = _(slotsDefinition)
                .filter(slot => this.filterByPlatform(slot))
                .map(slot => ({
                dataType: _.includes(slot.type, "@sys.") ? slot.type : `@${slot.type}`,
                name: slot.name,
                value: `$${slot.name}`,
                isList: false
            }))
                .value();
            const resultSamples = samples.map(sample => {
                const data = _.chain(sample)
                    .replace(/{([^}]+)}/g, (match, inner) => {
                    return `|{${inner}}|`;
                })
                    .split("|")
                    .map(text => {
                    const element = {};
                    const isTemplate = _.includes(text, "{") && _.includes(text, "}");
                    const alias = text.replace("{", "").replace("}", "");
                    const slot = _.find(parameters, { name: text });
                    if (isTemplate && slot) {
                        _.set(element, "meta", `@${_.kebabCase(slot.dataType)}`);
                        _.set(element, "alias", alias);
                    }
                    if (!_.isEmpty(text)) {
                        _.set(element, "text", text);
                        _.set(element, "userDefined", isTemplate);
                    }
                    _.set(element, "id", hashObj(element));
                    return _.isEmpty(_.omit(element, ["id"])) ? null : element;
                })
                    .compact()
                    .value();
                return {
                    data,
                    isTemplate: false,
                    count: 0,
                    updated: 0
                };
            });
            if (!_.isEmpty(resultSamples)) {
                const file = {
                    path: path.join(this.interactionOptions.rootPath, this.interactionOptions.speechPath, this.NAMESPACE, environment, "intents", `${name}_usersays_${locale}.json`),
                    content: resultSamples
                };
                this.fileContent.push(file);
            }
        });
    }
    buildIntent(locale, environment) {
        const intentsByPlatformAndEnvironments = this.intentsByPlatformAndEnvironments(locale, environment);
        locale = this.getLocale(locale);
        this.builtIntents = intentsByPlatformAndEnvironments.map((rawIntent) => {
            let { name, events } = rawIntent;
            const { webhookForSlotFilling, slotsDefinition, responses, webhookUsed } = rawIntent;
            name = name.replace("AMAZON.", "");
            const fallbackIntent = name === "FallbackIntent";
            const action = fallbackIntent ? "input.unknown" : name;
            events = name === "LaunchIntent" ? ["WELCOME", "GOOGLE_ASSISTANT_WELCOME"] : events;
            events = events.map((eventName) => ({ name: eventName }));
            const parameters = _(slotsDefinition)
                .filter(slot => this.filterByPlatform(slot))
                .map(slot => ({
                dataType: _.includes(slot.type, "@sys.") ? slot.type : `@${_.kebabCase(slot.type)}`,
                name: slot.name.replace("{", "").replace("}", ""),
                value: `$${slot.name.replace("{", "").replace("}", "")}`,
                isList: false
            }))
                .value();
            const messages = [];
            if (responses.length > 0) {
                messages.push({
                    type: 0,
                    lang: locale,
                    speech: responses
                });
            }
            const intent = {
                name,
                auto: true,
                contexts: [],
                responses: [
                    {
                        resetContexts: false,
                        action,
                        affectedContexts: [],
                        parameters,
                        messages,
                        defaultResponsePlatforms: {},
                        speech: []
                    }
                ],
                priority: 500000,
                webhookUsed,
                webhookForSlotFilling,
                fallbackIntent,
                events
            };
            _.set(intent, "id", hashObj(intent));
            const file = {
                path: path.join(this.interactionOptions.rootPath, this.interactionOptions.speechPath, this.NAMESPACE, environment, "intents", `${intent.name}.json`),
                content: intent
            };
            this.fileContent.push(file);
            return intent;
        });
    }
    buildEntities(locale, environment) {
        const localeEntity = this.getLocale(locale);
        this.getSlotsByIntentsDefinition(locale, environment)
            .filter(slot => !_.includes(slot.name, "@sys."))
            .forEach(rawSlot => {
            const { name, values } = rawSlot;
            const slotName = _.kebabCase(name);
            const slotContent = {
                name: slotName,
                isOverridable: true,
                isEnum: false,
                automatedExpansion: false
            };
            _.set(slotContent, "id", hashObj(slotContent));
            const fileDef = {
                path: path.join(this.interactionOptions.rootPath, this.interactionOptions.speechPath, this.NAMESPACE, environment, "entities", `${slotName}.json`),
                content: slotContent
            };
            const fileValue = {
                path: path.join(this.interactionOptions.rootPath, this.interactionOptions.speechPath, this.NAMESPACE, environment, "entities", `${slotName}_entries_${localeEntity}.json`),
                content: values
            };
            this.fileContent.push(fileDef, fileValue);
        });
    }
}
exports.DialogflowSchema = DialogflowSchema;
function hashObj(obj) {
    return uuid(JSON.stringify(obj), uuid.DNS);
}
//# sourceMappingURL=DialogflowSchema.js.map