"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const VoxaSheet_1 = require("./VoxaSheet");
function sheetLocale(voxaSheet, AVAILABLE_LOCALES) {
    let locale = AVAILABLE_LOCALES.find((loc) => {
        let lastDotIndex = _.lastIndexOf(voxaSheet.spreadsheetTitle, ".");
        lastDotIndex = lastDotIndex === -1 ? voxaSheet.spreadsheetTitle.length : lastDotIndex;
        const titleWithoutExtension = _.toLower(voxaSheet.spreadsheetTitle.slice(0, lastDotIndex));
        return _.endsWith(titleWithoutExtension, _.toLower(loc));
    });
    locale = locale || AVAILABLE_LOCALES[0];
    return locale;
}
exports.sheetLocale = sheetLocale;
function downloadProcessor(voxaSheets, AVAILABLE_LOCALES) {
    const voxaSheetsDownloads = voxaSheets.filter(voxaSheet => _.includes([VoxaSheet_1.SheetTypes.DOWNLOAD], VoxaSheet_1.getSheetType(voxaSheet)));
    return voxaSheetsDownloads.map((voxaSheet) => {
        const locale = sheetLocale(voxaSheet, AVAILABLE_LOCALES);
        const sheetPlaceholder = VoxaSheet_1.getSheetType(voxaSheet);
        const name = (voxaSheet.sheetTitle = voxaSheet.sheetTitle.replace(sheetPlaceholder, ""));
        const data = _.chain(voxaSheet.data)
            .filter(item => {
            const tempItem = _.omitBy(item, _.isEmpty);
            return !_.isEmpty(tempItem);
        })
            .value();
        const download = { name, data, locale };
        return download;
    });
}
exports.downloadProcessor = downloadProcessor;
function invocationProcessor(voxaSheets, AVAILABLE_LOCALES) {
    const voxaSheetsInvocations = voxaSheets.filter(voxaSheet => _.includes([VoxaSheet_1.SheetTypes.INVOCATION], VoxaSheet_1.getSheetType(voxaSheet)));
    return voxaSheetsInvocations.reduce((acc, voxaSheet) => {
        const locale = sheetLocale(voxaSheet, AVAILABLE_LOCALES);
        voxaSheet.data.map((item) => {
            const { environment, invocationName } = item;
            acc.push({ name: invocationName, environment, locale });
        });
        return acc;
    }, []);
}
exports.invocationProcessor = invocationProcessor;
function viewsProcessor(voxaSheets, AVAILABLE_LOCALES) {
    function sanitizeView(text = "") {
        return text
            .replace(/’/g, "'")
            .replace(/’/g, "'")
            .replace(/“/g, '"')
            .replace(/”/g, '"')
            .replace(/&/g, "and");
    }
    const voxaSheetsViews = voxaSheets.filter(voxaSheet => _.includes([VoxaSheet_1.SheetTypes.VIEWS], VoxaSheet_1.getSheetType(voxaSheet)));
    return voxaSheetsViews.map((voxaSheet) => {
        const locale = voxaSheet.sheetTitle.split("@")[1] || AVAILABLE_LOCALES[0];
        const data = _.chain(voxaSheet.data)
            .reduce((acc, view) => {
            const { path } = view;
            const pathLowerCase = _.toLower(path);
            let { value } = view;
            if (_.isEmpty(path)) {
                return acc;
            }
            const shouldBeArray = [".text", ".say", ".reprompt", ".tell", ".ask"].find(suffix => path.includes(suffix));
            const isASuggestionChip = [".dialogflowsuggestions", ".facebooksuggestionchips"].find(option => pathLowerCase.includes(option));
            if (shouldBeArray && _.isString(value) && !_.isEmpty(value)) {
                const temp = _.get(acc, path, []);
                temp.push(sanitizeView(value));
                value = temp;
            }
            if (!_.isEmpty(value) && isASuggestionChip) {
                value = value.split("\n").map((v) => v.trim());
            }
            _.set(acc, path, value);
            return acc;
        }, {})
            .value();
        const viewResult = { data, locale };
        return viewResult;
    });
}
exports.viewsProcessor = viewsProcessor;
function slotProcessor(voxaSheets, AVAILABLE_LOCALES) {
    const voxaSheetsSlots = voxaSheets.filter(voxaSheet => _.includes([VoxaSheet_1.SheetTypes.SLOTS], VoxaSheet_1.getSheetType(voxaSheet)));
    return voxaSheetsSlots.map((voxaSheet) => {
        const locale = sheetLocale(voxaSheet, AVAILABLE_LOCALES);
        const name = voxaSheet.sheetTitle;
        const values = _.chain(voxaSheet.data)
            .groupBy("synonym")
            .toPairs()
            .reduce((acc, slot) => {
            const key = slot[0];
            const synonyms = slot[1] || [];
            if (key === undefined || key === "undefined") {
                acc.push(synonyms.map(synonymName => ({ value: synonymName[name], synonyms: [] })));
            }
            else {
                acc.push({ value: key, synonyms: _.map(synonyms, name) });
            }
            return acc;
        }, [])
            .flattenDeep()
            .filter("value")
            .uniq()
            .value();
        const slotResult = { name, values, locale };
        return slotResult;
    });
}
exports.slotProcessor = slotProcessor;
function intentUtterProcessor(voxaSheets, AVAILABLE_LOCALES) {
    const voxaSheetsIntent = filterSheets(voxaSheets, [VoxaSheet_1.SheetTypes.INTENT]);
    const voxaSheetsUtter = _.reduce(filterSheets(voxaSheets, [VoxaSheet_1.SheetTypes.UTTERANCE]), reduceIntent("utterance"), []);
    const voxaSheetResponses = _.reduce(filterSheets(voxaSheets, [VoxaSheet_1.SheetTypes.RESPONSES]), reduceIntent("response"), []);
    const result = _.chain(voxaSheetsIntent)
        .map((voxaSheetIntent) => {
        const locale = sheetLocale(voxaSheetIntent, AVAILABLE_LOCALES);
        let previousIntent;
        voxaSheetIntent.data = _.chain(voxaSheetIntent.data)
            .map(row => {
            const info = _.pick(row, [
                "Intent",
                "slotType",
                "slotName",
                "environment",
                "platformIntent",
                "events",
                "canFulfillIntent",
                "startIntent",
                "signInRequired",
                "endIntent",
                "platformSlot",
                "webhookForSlotFilling",
                "webhookUsed"
            ]);
            previousIntent = _.isEmpty(info.Intent) ? previousIntent : info.Intent;
            info.Intent = previousIntent;
            return info;
        })
            .uniq()
            .groupBy("Intent")
            .toPairs()
            .reduce((acc, item) => {
            const intentName = item[0];
            const head = _.head(item[1]);
            const events = _.chain(head)
                .get("events", "")
                .split(",")
                .map(_.trim)
                .compact()
                .value();
            const signInRequired = _.get(head, "signInRequired", false);
            const environments = _.chain(head)
                .get("environment", "")
                .split(",")
                .map(_.trim)
                .compact()
                .value();
            const platforms = _.chain(head)
                .get("platformIntent", "")
                .split(",")
                .map(_.trim)
                .map(_.toLower)
                .compact()
                .value();
            const webhookForSlotFilling = (_.get(head, "webhookForSlotFilling", false) ||
                _.get(head, "useWebhookForSlotFilling", false));
            const webhookUsed = _.get(head, "webhookUsed", true);
            const canFulfillIntent = _.get(head, "canFulfillIntent", false);
            const startIntent = _.get(head, "startIntent", false);
            const endIntent = _.get(head, "endIntent", false);
            const samples = getIntentValueList(voxaSheetsUtter, voxaSheetIntent.spreadsheetId, intentName, "utterance");
            const responses = getIntentValueList(voxaSheetResponses, voxaSheetIntent.spreadsheetId, intentName, "response");
            const slotsDefinition = _.chain(item[1])
                .filter("slotName")
                .map((slot) => ({
                name: slot.slotName,
                type: slot.slotType,
                platform: slot.platformSlot
            }))
                .compact()
                .uniq()
                .value();
            const intent = {
                name: intentName,
                samples,
                responses,
                slotsDefinition,
                webhookForSlotFilling,
                webhookUsed,
                canFulfillIntent,
                startIntent,
                endIntent,
                events,
                environments,
                platforms,
                locale,
                signInRequired
            };
            acc.push(intent);
            return acc;
        }, [])
            .value();
        return voxaSheetIntent.data;
    })
        .flattenDeep()
        .value();
    return result;
}
exports.intentUtterProcessor = intentUtterProcessor;
function publishingProcessor(voxaSheets, AVAILABLE_LOCALES) {
    const voxaSheetsPublishing = voxaSheets.filter(voxaSheet => _.includes([
        VoxaSheet_1.SheetTypes.SKILL_ENVIRONMENTS,
        VoxaSheet_1.SheetTypes.SKILL_LOCALE_INFORMATION,
        VoxaSheet_1.SheetTypes.SKILL_GENERAL_INFORMATION
    ], VoxaSheet_1.getSheetType(voxaSheet)));
    return voxaSheetsPublishing.reduce((acc, voxaSheet) => {
        voxaSheet.data.map((item) => {
            const locale = voxaSheet.sheetTitle.split("@")[1] || AVAILABLE_LOCALES[0];
            const environments = _.chain(item)
                .get("environment", "")
                .split(",")
                .map(_.trim)
                .compact()
                .value();
            const key = _.chain(item)
                .get("key", "")
                .replace("{locale}", locale)
                .value();
            const value = _.chain(item)
                .get("value", "")
                .value();
            const publishInfo = { key, value, environments };
            acc.push(publishInfo);
        });
        return acc;
    }, []);
}
exports.publishingProcessor = publishingProcessor;
function filterSheets(voxaSheets, sheetTypes) {
    return _.filter(voxaSheets, voxaSheet => _.includes(sheetTypes, VoxaSheet_1.getSheetType(voxaSheet)));
}
function reduceIntent(propName) {
    return (acc, row) => {
        row.data = _.chain(row.data)
            .reduce((accData, item) => {
            _.map(item, (value, key) => {
                const obj = { intent: key };
                obj[propName] = value;
                accData.push(obj);
            });
            return accData;
        }, [])
            .groupBy("intent")
            .value();
        acc.push(row);
        return acc;
    };
}
function getIntentValueList(voxaSheets, spreadsheetId, intentName, key) {
    return _(voxaSheets)
        .filter(sheet => sheet.spreadsheetId === spreadsheetId)
        .map((spreadSheet) => spreadSheet.data[intentName] || [])
        .flatten()
        .map(key)
        .compact()
        .uniq()
        .value();
}
//# sourceMappingURL=Processor.js.map