/*
 * Copyright (c) 2018 Rain Agency <contact@rain.agency>
 * Author: Rain Agency <contact@rain.agency>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* tslint:disable:no-submodule-imports no-console */
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const bluebird_1 = require("bluebird");
const colors = require("colors");
const fs = require("fs-extra");
const inquirer = require("inquirer");
const _ = require("lodash");
const pad = require("pad");
const path = require("path");
const rxjs_1 = require("rxjs");
const InteractionBuilder_1 = require("../InteractionBuilder");
exports.name = "init";
exports.alias = "";
exports.description = "create a new interaction.json";
const ALLOWED_ATTRIBUTES = [
    "platforms",
    "spreadsheets",
    "speechPath",
    "content",
    "contentPath",
    "viewsPath"
];
function action() {
    return __awaiter(this, void 0, void 0, function* () {
        const interactionFile = path.join(process.cwd(), "interaction.json");
        let hasAnInteractionFile = false;
        try {
            // a path we KNOW is totally bogus and not a module
            hasAnInteractionFile = require(interactionFile);
        }
        catch (e) {
            if (e.code === "MODULE_NOT_FOUND") {
                hasAnInteractionFile = false;
            }
        }
        const observe = rxjs_1.Observable.create((obs) => {
            const override = (answers) => {
                return answers.interactionFile;
            };
            const onlyCommaAnswer = (input) => {
                return _.chain(input)
                    .split(",")
                    .map(_.trim)
                    .compact()
                    .value();
            };
            const noEmptyAnswer = (input) => !_.isEmpty(input);
            const mustBeSpreadsheetURLOrLength30 = (input) => __awaiter(this, void 0, void 0, function* () {
                let result = _.chain(input)
                    .split(",")
                    .map(_.trim)
                    .compact()
                    .map((i) => {
                    if (i.includes("docs.google.com/spreadsheets")) {
                        const matched = i.match(/docs\.google\.com\/spreadsheets\/d\/(.*)\/.*/);
                        return matched && _.isString(matched[1]) && matched[1].length > 30;
                    }
                    if (i.includes(".sharepoint.com") && i.length > 30) {
                        return true;
                    }
                    const customPath = i.indexOf("/") === 0 ? i : path.join(process.cwd(), i);
                    return fs.pathExists(customPath);
                })
                    .value();
                result = yield bluebird_1.all(result);
                result = result.every((r) => r);
                return noEmptyAnswer(input) && result
                    ? true
                    : "Insert a proper google spreadsheet url, Office 365 sharepoint url, local file or local folder path. e.g. https://docs.google.com/spreadsheets/d/XXXXXXX/edit#gid=0, https://XXXXXX.sharepoint.com/:x:/g/personal/YYYYYY/ZZZZZZZ, /Users/local/file.xlsx or /Users/local/sheets/";
            });
            const questions = [
                {
                    type: "confirm",
                    name: "interactionFile",
                    message: `Do you want to override ${interactionFile}?`,
                    when: () => hasAnInteractionFile,
                    default: true
                },
                {
                    type: "checkbox",
                    name: "platforms",
                    message: "Choose platforms",
                    choices: ["alexa", "dialogflow"],
                    when: override
                },
                {
                    type: "input",
                    name: "spreadsheets",
                    message: "Specify all spreadsheets separated by comma (*)",
                    validate: mustBeSpreadsheetURLOrLength30,
                    filter: onlyCommaAnswer,
                    when: override
                },
                {
                    type: "input",
                    name: "speechPath",
                    message: "Specify folder path to save interaction model and manifest",
                    default: InteractionBuilder_1.DEFAULT_INTERACTION_OPTIONS.speechPath,
                    when: override
                },
                {
                    type: "input",
                    name: "contentPath",
                    message: "Specify folder path to save all downloable content",
                    when: override,
                    default: InteractionBuilder_1.DEFAULT_INTERACTION_OPTIONS.contentPath
                },
                {
                    type: "input",
                    name: "viewsPath",
                    message: "Specify folder path to save views file",
                    default: InteractionBuilder_1.DEFAULT_INTERACTION_OPTIONS.viewsPath,
                    when: override
                },
                {
                    type: "confirm",
                    name: "isCorrect",
                    message: `is Correct?`,
                    when: (answers) => {
                        if (override(answers)) {
                            printVariables(_.cloneDeep(answers));
                        }
                        return override(answers);
                    },
                    default: true
                }
            ];
            questions.map(q => obs.next(q));
            obs.complete();
        });
        function printVariables(answers) {
            return _.chain(answers)
                .toPairs()
                .filter(item => !!ALLOWED_ATTRIBUTES.find(i => i === item[0]))
                .map(item => {
                const key = item[0];
                const value = item[1];
                console.log(pad(colors.blue(key), 20), colors.grey(value));
                return item;
            })
                .compact()
                .fromPairs()
                .value();
        }
        function executePrompt() {
            return __awaiter(this, void 0, void 0, function* () {
                return inquirer.prompt(observe).then((answers) => {
                    if (answers.interactionFile && !answers.isCorrect) {
                        return executePrompt();
                    }
                    if (hasAnInteractionFile && !answers.interactionFile) {
                        answers = hasAnInteractionFile;
                    }
                    if (!answers.interactionFile && !answers.isCorrect) {
                        printVariables(answers);
                    }
                    answers = _.pick(answers, ALLOWED_ATTRIBUTES);
                    return fs.outputFile(path.join(process.cwd(), "interaction.json"), JSON.stringify(answers, null, 2), { flag: "w" });
                });
            });
        }
        yield executePrompt();
    });
}
exports.action = action;
//# sourceMappingURL=init.js.map